module doorSafetyProblem
-- generic modules
open events
open domains

-- domains
one sig Operators extends Domain { }
one sig Door extends Domain { }
one sig PowerSource extends Domain {
	Exposed, Live: set Time
	} {
	Switch in OK implies {
		not Live.initially
		not Exposed.initially
		}
	}
one sig Sensor extends Domain {
	Closed: set Time
	}
one sig Switch extends Domain { }
one sig Controller extends Domain { } {
-- off occurs if Closed became false
	this in OK implies {
		all t: Time - (first + last) |
			not Sensor.Closed.at[t] and Sensor.Closed.at[t.prev] implies Off.happensAt [t]
		}
	}

-- events
sig Open extends Event { } {
	Sensor in OK implies {
		not Sensor.Closed.after
		}
	}
sig Close extends Event { } {
	Door in OK implies {
		not PowerSource.Exposed.after
		}
	}
sig On extends Event { } {
	Controller in OK implies {
		Sensor.Closed.before
		}
	}
sig Off extends Event { } {
	Switch in OK implies {
		not PowerSource.Live.after
		}
	}	
sig Touch extends Event { } {
	-- which domain here?
	Door in OK implies {
		PowerSource.Exposed.before
		}
	-- in the previous time step, an Open was not executed
	-- commented out to show counterexample
	--	pre.prev not in Open.@pre
	-- better formulation
	Operators in OK implies {
--		no o: Open | this.follows [o]
		}
	}
-- time passes without any effect
sig Stutter extends Event { }

-- assumptions about non-effects of other events
sig Frame in Event {} {
	changed [Closed] implies this in Open + Close
	changed [Exposed] implies this in Open + Close
	changed [Live] implies this in On + Off
	}
fact { Door + Switch in OK implies Event in Frame }

one sig Safe extends Requirement {} {
	this in OK iff all t: Touch | not PowerSource.Live.before [t]
	trustedBase = Switch + Controller + Sensor + Door + Operators
	}

-- check BaseSufficient for 3
check BaseSufficient for 3

